@@ -24,6 +24,9 @@
 #include <fluent-bit/flb_info.h>
 #include <fluent-bit/flb_output.h>
 #include <fluent-bit/flb_log.h>
+#include <cmetrics/cmetrics.h>
+#include <cmetrics/cmt_encode_text.h>
+#include <cmetrics/cmt_encode_prometheus.h>
 
 #define flb_plg_error(ctx, fmt, ...)                                    \
     if (flb_log_check_level(ctx->log_level, FLB_LOG_ERROR))             \
