@@ -29,8 +29,7 @@
 // FIXME: Revise this documentation:
 /**
  * This file implements the consumer offset storage.
- * It currently supports local file storage and broker OffsetCommit storage,
- * not zookeeper.
+ * It currently supports local file storage and broker OffsetCommit storage.
  *
  * Regardless of commit method (file, broker, ..) this is how it works:
  *  - When rdkafka, or the application, depending on if auto.offset.commit
@@ -511,7 +510,8 @@ rd_kafka_offset_broker_commit_cb (rd_kafka_t *rk,
 
         rktpar = &offsets->elems[0];
 
-        if (!(rktp = rd_kafka_topic_partition_list_get_toppar(rk, rktpar))) {
+        if (!(rktp = rd_kafka_topic_partition_get_toppar(rk, rktpar,
+                                                         rd_false))) {
 		rd_kafka_dbg(rk, TOPIC, "OFFSETCOMMIT",
 			     "No local partition found for %s [%"PRId32"] "
 			     "while parsing OffsetCommit response "
@@ -529,9 +529,10 @@ rd_kafka_offset_broker_commit_cb (rd_kafka_t *rk,
 	rd_kafka_toppar_offset_commit_result(rktp, err, offsets);
 
         rd_kafka_dbg(rktp->rktp_rkt->rkt_rk, TOPIC, "OFFSET",
-                     "%s [%"PRId32"]: offset %"PRId64" committed: %s",
+                     "%s [%"PRId32"]: offset %"PRId64" %scommitted: %s",
                      rktp->rktp_rkt->rkt_topic->str,
                      rktp->rktp_partition, rktpar->offset,
+                     err ? "not " : "",
                      rd_kafka_err2str(err));
 
         rktp->rktp_committing_offset = 0;
@@ -677,7 +678,8 @@ rd_kafka_offsets_store (rd_kafka_t *rk,
                 rd_kafka_topic_partition_t *rktpar = &offsets->elems[i];
                 rd_kafka_toppar_t *rktp;
 
-                rktp = rd_kafka_topic_partition_get_toppar(rk, rktpar);
+                rktp = rd_kafka_topic_partition_get_toppar(rk, rktpar,
+                                                           rd_false);
                 if (!rktp) {
                         rktpar->err = RD_KAFKA_RESP_ERR__UNKNOWN_PARTITION;
                         continue;
@@ -773,8 +775,9 @@ void rd_kafka_offset_reset (rd_kafka_toppar_t *rktp, int64_t err_offset,
 	if (offset == RD_KAFKA_OFFSET_INVALID) {
 		/* Error, auto.offset.reset tells us to error out. */
                 rd_kafka_consumer_err(rktp->rktp_fetchq, RD_KAFKA_NODEID_UA,
-                                      err, 0, NULL, rktp, err_offset,
-                                      "%s", reason);
+                                      RD_KAFKA_RESP_ERR__AUTO_OFFSET_RESET,
+                                      0, NULL, rktp, err_offset,
+                                      "%s: %s", reason, rd_kafka_err2str(err));
                 rd_kafka_toppar_set_fetch_state(
 			rktp, RD_KAFKA_TOPPAR_FETCH_NONE);
 
