@@ -63,6 +63,22 @@ static uint64_t time_ms_now()
     return ms;
 }
 
+
+int flb_ml_flush_stdout(struct flb_ml_parser *parser,
+                        struct flb_ml_stream *mst,
+                        void *data, char *buf_data, size_t buf_size)
+{
+    fprintf(stdout, "\n%s----- MULTILINE FLUSH (stream_id=%" PRIu64 ") -----%s\n",
+            ANSI_GREEN, mst->id, ANSI_RESET);
+
+    /* Print incoming flush buffer */
+    flb_pack_print(buf_data, buf_size);
+
+    fprintf(stdout, "%s----------- EOF -----------%s\n",
+            ANSI_GREEN, ANSI_RESET);
+    return 0;
+}
+
 int flb_ml_type_lookup(char *str)
 {
     int type = -1;
@@ -122,6 +138,14 @@ void flb_ml_flush_pending(struct flb_ml *ml, uint64_t now)
     }
 }
 
+void flb_ml_flush_pending_now(struct flb_ml *ml)
+{
+    uint64_t now;
+
+    now = time_ms_now();
+    flb_ml_flush_pending(ml, now);
+}
+
 static void cb_ml_flush_timer(struct flb_config *ctx, void *data)
 {
     uint64_t now;
@@ -416,8 +440,8 @@ static int process_append(struct flb_ml_parser_ins *parser_i,
     else if (type == FLB_ML_TYPE_MAP) {
         full_map = obj;
         if (!full_map) {
-            msgpack_unpacked_init(&result);
             off = 0;
+            msgpack_unpacked_init(&result);
             ret = msgpack_unpack_next(&result, buf, size, &off);
             if (ret != MSGPACK_UNPACK_SUCCESS) {
                 return -1;
@@ -500,10 +524,11 @@ static int ml_append_try_parser(struct flb_ml_parser_ins *parser,
         /* Parse incoming content */
         ret = flb_parser_do(parser->ml_parser->parser, (char *) buf, size,
                             &out_buf, &out_size, &out_time);
+        if (flb_time_to_double(&out_time) == 0.0) {
+            flb_time_copy(&out_time, tm);
+        }
+
         if (ret >= 0) {
-            if (flb_time_to_double(&out_time) == 0.0) {
-                flb_time_copy(&out_time, tm);
-            }
             release = FLB_TRUE;
             type = FLB_ML_TYPE_MAP;
         }
@@ -597,7 +622,8 @@ int flb_ml_append(struct flb_ml *ml, uint64_t stream_id,
 
     mk_list_foreach(head_group, &group->parsers) {
             parser_i = mk_list_entry(head_group, struct flb_ml_parser_ins, _head);
-            if (lru_parser && parser_i == lru_parser) {
+            if (lru_parser && lru_parser == parser_i &&
+                lru_parser->last_stream_id == stream_id) {
                 continue;
             }
 
@@ -613,7 +639,6 @@ int flb_ml_append(struct flb_ml *ml, uint64_t stream_id,
             else {
                 parser_i = NULL;
             }
-
     }
 
     if (!processed) {
@@ -751,6 +776,7 @@ int flb_ml_append_object(struct flb_ml *ml, uint64_t stream_id,
 
         /* Append record content to group msgpack buffer */
         msgpack_pack_array(&st_group->mp_pck, 2);
+
         flb_time_append_to_msgpack(tm, &st_group->mp_pck, 0);
         msgpack_pack_object(&st_group->mp_pck, *obj);
 
@@ -898,6 +924,11 @@ int flb_ml_flush_stream_group(struct flb_ml_parser *ml_parser,
     msgpack_sbuffer_init(&mp_sbuf);
     msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
 
+    /* if the group don't have a time set, use current time */
+    if (flb_time_to_double(&group->mp_time) == 0.0) {
+        flb_time_get(&group->mp_time);
+    }
+
     /* compose final record if we have a first line context */
     if (group->mp_sbuf.size > 0) {
         msgpack_unpacked_init(&result);
