@@ -307,6 +307,41 @@ struct record_check go_output[] = {
     {"one more line, no multiline\n"}
 };
 
+/*
+ * Issue 3817 (case: 1)
+ * --------------------
+ * Source CRI messages (need first CRI multiline parsing) + a custom multiline
+ * parser.
+ *
+ *   - https://github.com/fluent/fluent-bit/issues/3817
+ *
+ * The 'case 1' represents the problems of identifying two consecutive multiline
+ * messages within the same stream.
+ */
+struct record_check issue_3817_1_input[] = {
+    {"2021-05-17T17:35:01.184675702Z stdout F [DEBUG] 1 start multiline - "},
+    {"2021-05-17T17:35:01.184747208Z stdout F 1 cont A"},
+    {"2021-05-17T17:35:01.184675702Z stdout F [DEBUG] 2 start multiline - "},
+    {"2021-05-17T17:35:01.184747208Z stdout F 2 cont B"},
+    {"another isolated line"}
+};
+
+struct record_check issue_3817_1_output[] = {
+    {
+      "[DEBUG] 1 start multiline - \n"
+      "1 cont A"
+    },
+
+    {
+      "[DEBUG] 2 start multiline - \n"
+      "2 cont B"
+    },
+
+    {
+      "another isolated line"
+    }
+};
+
 /*
  * Flush callback is invoked every time a multiline stream has completed a multiline
  * message or a message is not multiline.
@@ -344,6 +379,8 @@ static int flush_callback(struct flb_ml_parser *parser,
 
     flb_time_pop_from_msgpack(&tm, &result, &map);
 
+    TEST_CHECK(flb_time_to_double(&tm) != 0.0);
+
     exp = &res->out_records[res->current_record];
     len = strlen(res->key);
     for (i = 0; i < map->via.map.size; i++) {
@@ -554,6 +591,8 @@ static void test_parser_java()
     struct flb_ml *ml;
     struct flb_ml_parser_ins *mlp_i;
     struct expected_result res = {0};
+    msgpack_packer mp_pck;
+    msgpack_sbuffer mp_sbuf;
 
     /* Expected results context */
     res.key = "log";
@@ -570,10 +609,21 @@ static void test_parser_java()
     mlp_i = flb_ml_parser_instance_create(ml, "java");
     TEST_CHECK(mlp_i != NULL);
 
+    flb_ml_parser_instance_set(mlp_i, "key_content", "log");
+
     ret = flb_ml_stream_create(ml, "java", -1, flush_callback, (void *) &res,
                                &stream_id);
     TEST_CHECK(ret == 0);
 
+    /* initialize buffers */
+    msgpack_sbuffer_init(&mp_sbuf);
+    msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
+
+    size_t off = 0;
+    msgpack_unpacked result;
+    msgpack_object root;
+    msgpack_object *map;
+
     entries = sizeof(java_input) / sizeof(struct record_check);
     for (i = 0; i < entries; i++) {
         r = &java_input[i];
@@ -581,7 +631,35 @@ static void test_parser_java()
 
         /* Package as msgpack */
         flb_time_get(&tm);
-        flb_ml_append(ml, stream_id, FLB_ML_TYPE_TEXT, &tm, r->buf, len);
+
+        /* initialize buffers */
+        msgpack_sbuffer_init(&mp_sbuf);
+        msgpack_packer_init(&mp_pck, &mp_sbuf, msgpack_sbuffer_write);
+
+        msgpack_pack_array(&mp_pck, 2);
+        flb_time_append_to_msgpack(&tm, &mp_pck, 0);
+
+        msgpack_pack_map(&mp_pck, 1);
+        msgpack_pack_str(&mp_pck, 3);
+        msgpack_pack_str_body(&mp_pck, "log", 3);
+        msgpack_pack_str(&mp_pck, len);
+        msgpack_pack_str_body(&mp_pck, r->buf, len);
+
+        /* Unpack and lookup the content map */
+        msgpack_unpacked_init(&result);
+        off = 0;
+        ret = msgpack_unpack_next(&result, mp_sbuf.data, mp_sbuf.size, &off);
+
+        flb_pack_print(mp_sbuf.data, mp_sbuf.size);
+
+        root = result.data;
+        map = &root.via.array.ptr[1];
+
+        /* Package as msgpack */
+        ret = flb_ml_append_object(ml, stream_id, &tm, map);
+
+        msgpack_unpacked_destroy(&result);
+        msgpack_sbuffer_destroy(&mp_sbuf);
     }
 
     if (ml) {
@@ -875,7 +953,183 @@ static void test_parser_go()
     flb_config_exit(config);
 }
 
+static int flush_callback_to_buf(struct flb_ml_parser *parser,
+                                 struct flb_ml_stream *mst,
+                                 void *data, char *buf_data, size_t buf_size)
+{
+    msgpack_sbuffer *mp_sbuf = data;
+    msgpack_sbuffer_write(mp_sbuf, buf_data, buf_size);
+
+    return 0;
+}
+
+static void run_test(struct flb_config *config, char *test_name,
+                     struct record_check *in, int in_len,
+                     struct record_check *out, int out_len,
+                     char *parser1, char *parser2)
+
+{
+    int i;
+    int ret;
+    int len;
+    size_t off = 0;
+    uint64_t stream1 = 0;
+    uint64_t stream2 = 0;
+    struct flb_ml *ml;
+    struct flb_ml_parser_ins *p1 = NULL;
+    struct record_check *r;
+    msgpack_sbuffer mp_sbuf1;
+    msgpack_packer mp_pck1;
+    msgpack_sbuffer mp_sbuf2;
+    msgpack_packer mp_pck2;
+    msgpack_object *map;
+    struct flb_time tm;
+    struct expected_result res = {0};
+    msgpack_unpacked result;
+
+    /* init buffers */
+    msgpack_sbuffer_init(&mp_sbuf1);
+    msgpack_packer_init(&mp_pck1, &mp_sbuf1, msgpack_sbuffer_write);
+    msgpack_sbuffer_init(&mp_sbuf2);
+    msgpack_packer_init(&mp_pck2, &mp_sbuf2, msgpack_sbuffer_write);
+
+    /* Create docker multiline mode */
+    ml = flb_ml_create(config, test_name);
+    TEST_CHECK(ml != NULL);
+
+    if (!parser1) {
+        fprintf(stderr, "run_test(): parser1 is NULL\n");
+        exit(1);
+    }
+
+    /* Parser 1 */
+    p1 = flb_ml_parser_instance_create(ml, parser1);
+    TEST_CHECK(p1 != NULL);
+
+
+    /* Stream 1: use parser name (test_name) to generate the stream id */
+    ret = flb_ml_stream_create(ml, test_name, -1,
+                               flush_callback_to_buf,
+                               (void *) &mp_sbuf1, &stream1);
+    TEST_CHECK(ret == 0);
+
+    /* Ingest input records into parser 1 */
+    for (i = 0; i < in_len; i++) {
+        r = &in[i];
+        len = strlen(r->buf);
+
+        flb_time_get(&tm);
+
+        /* Package as msgpack */
+        flb_ml_append(ml, stream1, FLB_ML_TYPE_TEXT, &tm, r->buf, len);
+    }
+
+    flb_ml_destroy(ml);
+    ml = flb_ml_create(config, test_name);
+
+    flb_ml_parser_instance_create(ml, parser2);
+
+    /*
+     * After flb_ml_append above(), mp_sbuf1 has been populated with the
+     * output results as structured messages. Now this data needs to be
+     * passed to the next parser.
+     */
+
+    /* Expected results context */
+    res.key = "log";
+    res.out_records = out;
+
+    /* Stream 2 */
+    ret = flb_ml_stream_create(ml, "filter_multiline", -1,
+                               flush_callback,
+                               (void *) &res, &stream2);
+
+    /* Ingest input records into parser 2 */
+    off = 0;
+    msgpack_unpacked_init(&result);
+    while (msgpack_unpack_next(&result, mp_sbuf1.data, mp_sbuf1.size, &off)) {
+        flb_time_pop_from_msgpack(&tm, &result, &map);
+
+        /* Package as msgpack */
+        ret = flb_ml_append_object(ml, stream2, &tm, map);
+    }
+    flb_ml_flush_pending_now(ml);
+
+    msgpack_unpacked_destroy(&result);
+    msgpack_sbuffer_destroy(&mp_sbuf1);
+    flb_ml_destroy(ml);
+}
+
+void test_issue_3817_1()
+{
+    int ret;
+    int in_len  = sizeof(issue_3817_1_input) / sizeof(struct record_check);
+    int out_len = sizeof(issue_3817_1_output) / sizeof(struct record_check);
+    struct flb_config *config;
+    struct flb_ml_parser *mlp;
+
+    /*
+     * Parser definition for a file:
+     *
+     * [MULTILINE_PARSER]
+     *     name           parser_3817
+     *     type           regex
+     *     key_content    log
+     *     #
+     *     # Regex rules for multiline parsing
+     *     # ---------------------------------
+     *     #
+     *     # rules |   state name  | regex pattern       | next state
+     *     # ------|---------------|------------------------------------
+     *     rule      "start_state"   "/- $/"                "cont"
+     *     rule      "cont"          "/^([1-9].*$/"         "cont"
+     *
+     */
+
+    /* Initialize environment */
+    config = flb_config_init();
+
+    /* Register custom parser */
+    mlp = flb_ml_parser_create(config,
+                               "parser_3817",        /* name      */
+                               FLB_ML_REGEX,         /* type      */
+                               NULL,                 /* match_str */
+                               FLB_FALSE,            /* negate    */
+                               1000,                 /* flush_ms  */
+                               "log",                /* key_content */
+                               NULL,                 /* key_pattern */
+                               NULL,                 /* key_group */
+                               NULL,                 /* parser ctx */
+                               NULL);                /* parser name */
+    TEST_CHECK(mlp != NULL);
+
+    /* rule: start_state */
+    ret = flb_ml_rule_create(mlp, "start_state", "/- $/", "cont", NULL);
+    if (ret != 0) {
+        fprintf(stderr, "error creating rule 1");
+    }
+
+    /* rule: cont */
+    ret = flb_ml_rule_create(mlp, "cont", "/^([1-9]).*$/", "cont", NULL);
+    if (ret != 0) {
+        fprintf(stderr, "error creating rule 2");
+    }
+
+    /* initiaze the parser configuration */
+    ret = flb_ml_parser_init(mlp);
+    TEST_CHECK(ret == 0);
+
+    /* Run the test */
+    run_test(config, "issue_3817_1",
+             issue_3817_1_input, in_len,
+             issue_3817_1_output, out_len,
+             "cri", "parser_3817");
+
+    flb_config_exit(config);
+}
+
 TEST_LIST = {
+    /* Normal features tests */
     { "parser_docker",  test_parser_docker},
     { "parser_cri",     test_parser_cri},
     { "parser_java",    test_parser_java},
@@ -884,5 +1138,8 @@ TEST_LIST = {
     { "parser_go",      test_parser_go},
     { "container_mix",  test_container_mix},
     { "endswith",       test_endswith},
+
+    /* Issues reported on Github */
+    { "issue_3817_1"  , test_issue_3817_1},
     { 0 }
 };
